unit IDGNumberEdit;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, About, DsgnIntf;

type
  TIDGNumberEdit = class(TCustomEdit)
  private
    FMask : string;
    FValue : double;
    FAddChar : string;

    procedure SetValue(Value : double);

  protected
    procedure CreateParams(var Params: TCreateParams); override;

    procedure DoExit; override;
    procedure DoEnter; override;

    procedure KeyPress(var Key: Char); override;

    procedure Change;override;

  public
    constructor Create(AOwner : TComponent);override;
  published
    property Mask : string read FMask write FMask;
    property Value : double read FValue write SetValue;

    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;

  end;

  function GetNumber( Value : string; var Number : double ) : boolean;

procedure Register;

implementation

function GetNumber( Value : string; var Number : double ) : boolean;
var
  i : Integer;
  sNumber : string;
begin
   Result := true;
   for i := 1 to Length(Value) do begin
    case Value[i] of
      '0'..'9' : sNumber := sNumber + Value[i];
      '+','-'  : begin
        if i = 1 then
          sNumber := sNumber + Value[i]
        else
          Result := false;
    end;
    else
      if Value[i] = DecimalSeparator then
        sNumber := sNumber + Value[i]
      else if not ( Value[i] = ThousandSeparator ) then
        Result := false;
    end;
  end;

  Number := StrToFloat(sNumber);

end;


constructor TIDGNumberEdit.Create(AOwner : TComponent);
begin
  inherited;
  Value := 0;
  FAddChar := '';
end;

procedure TIDGNumberEdit.SetValue(Value : double);
begin
  FValue := Value;
  Text := FormatFloat(FMask, FValue);
end;


procedure TIDGNumberEdit.Change;
begin
  inherited;
{  i := 0;
  if Parent <> nil then begin
    i := GetSelStart;
    OldLength := Length(Text);
  end;

  if GetNumber(Text, FValue ) then
    Text := FormatFloat(Mask, FValue);

  if i > 0 then begin
    delta := Length(Text) - OldLength;
    SetSelStart(i + delta);
  end;                     }

end;

procedure TIDGNumberEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style + ES_RIGHT;
end;

procedure TIDGNumberEdit.DoExit;
begin
  inherited;
  GetNumber(Text, FValue );
  Text := FormatFloat(Mask, FValue);
end;

procedure TIDGNumberEdit.DoEnter;
begin
  Text := FloatToStr(FValue);
  inherited;
  SelectAll;
end;


procedure TIDGNumberEdit.KeyPress(var Key: Char);
var
  lAbort : boolean;
  i : Integer;
begin

  lAbort := true;
  case Key of
    '0'..'9' :  lAbort := false;
  end;
  if Key = DecimalSeparator then
    lAbort := false;

  if Ord(Key) = 8 then
    lAbort := false;   

  i := GetSelStart;
  if (i = 0) and ((Key = '-') or (Key = '+')) then
    lAbort := false;

  if lAbort then begin
    Beep;
    Abort;
  end;

end;



procedure Register;
begin
  RegisterComponents('IDG', [TIDGNumberEdit]);
  RegisterComponentEditor( TIDGNumberEdit, TAboutEditor );
end;




end.
